module IFCFG
  class OVS
    Base = '/etc/sysconfig/network-scripts/ifcfg-'

    def self.remove(name)
      File.delete(Base + name)
    rescue Errno::ENOENT
    end

    def self.exists?(name)
      File.exist?(Base + name)
    end

    def initialize(name)
      @name        = name
      @device_type = 'ovs'
      @onboot      = 'yes'
    end

    def ifcfg
      { 'DEVICE'       => @name,
        'DEVICETYPE'   => @device_type,
        'TYPE'         => @type,
        'ONBOOT'       => @onboot,
        'OVSBOOTPROTO' => @bootproto }
    end

    def replace_or_add(obj, regexp, str)
      if obj.scan(regexp) == []
        obj << str << "\n"
      else
        obj.gsub(regexp, str)
      end
    end

    def file_to_str(filename)
      data = ''
      if File.exists?(filename)
        File.open(filename, 'r').each_line { |line| data << line }
      end
      data
    end

    def save
      fname = Base + @name
      str = file_to_str(fname)
      ifcfg.each { |key, val|
        str = replace_or_add(str, /^#{key}=.*$/, "#{key}=#{val}")
      }
      File.open(fname, 'w') { |file| file << str }
    end

    def to_s
      res = ''
      ifcfg.each { |x, y|
        res << "#{x}=#{y}\n"
      }
      res
    end
  end

  class Bridge < OVS
    def initialize(name, bootproto = nil)
      super(name)
      @type      = 'OVSBridge'
      @bootproto = bootproto ? bootproto : 'none'
    end
  end

  class BridgeDynamic < Bridge
    def initialize(name, interface, bridge_mac_address=nil)
      super(name, 'dhcp')
      @interface = interface
      @bridge_mac_address = bridge_mac_address
    end

    def ifcfg
      cfg = super.merge!({ 'OVSDHCPINTERFACES' => @interface })
      if @bridge_mac_address
        cfg.merge!({ 'OVS_EXTRA' =>
        "\"set bridge #{@name} other-config:hwaddr=#{@bridge_mac_address}\"" })
      end
      fname = Base + @interface
      str = file_to_str(fname)
      arr = ["PEERDNS", "DEFROUTE", "PEERROUTES", "DNS1", "DNS2", "MTU", "NM_CONTROLLED", "NOZEROCONF"]
      arr.each { |a|
        str.match(/^#{a}=.*$/) { |m|
          val = m.to_s.split('=',2)[1]
          cfg.merge!({ "#{a}" => "#{val}" })
        }
      }
      cfg
    end
  end

  class BridgeStatic < Bridge
    def initialize(name, cidr)
      super(name)
      cidr.match('(.*)\/(.*)') { |m|
        @ipaddr = m[1]
        @prefix = m[2]
      }
    end

    def ifcfg
      cfg = super
      if @cidr != ''
        cfg.merge!({ 'IPADDR' => @ipaddr, 'PREFIX' => @prefix })
      end
      fname = Base + @interface
      str = file_to_str(fname)
      arr = ["PEERDNS", "DEFROUTE", "PEERROUTES", "DNS1", "DNS2", "MTU", "NM_CONTROLLED", "NOZEROCONF"]
      arr.each { |a|
        str.match(/^#{a}=.*$/) { |m|
          val = m.to_s.split('=',2)[1]
          cfg.merge!({ "#{a}" => "#{val}" })
        }
      }
      cfg
    end
  end

  class Port < OVS
    def initialize(name, bridge)
      super(name)
      @type      = 'OVSPort'
      @bridge    = bridge
      @bootproto = 'none'
    end

    def ifcfg
      super.merge!({ 'BOOTPROTO' => 'none', 'OVS_BRIDGE' => @bridge })
    end
  end
end
